#!/usr/bin/env python
# coding:utf-8
"""
@author: nivic ybyang7
@license: Apache Licence
@file: wrapper.py
@time: 2022/06/16
@contact: ybyang7@iflytek.com
@site:
@software: PyCharm

"""

#  Copyright (c) 2022. Lorem ipsum dolor sit amet, consectetur adipiscing elit.
#  Morbi non lorem porttitor neque feugiat blandit. Ut vitae ipsum eget quam lacinia accumsan.
#  Etiam sed turpis ac ipsum condimentum fringilla. Maecenas magna.
#  Proin dapibus sapien vel ante. Aliquam erat volutpat. Pellentesque sagittis ligula eget metus.
#  Vestibulum commodo. Ut rhoncus gravida arcu.
import sys

try:
    from aiges_embed import ResponseData, Response, DataListNode, DataListCls,SessionCreateResponse  # c++
except:
    from aiges.dto import Response, ResponseData, DataListNode, DataListCls,SessionCreateResponse


from aiges.sdk import SessionManager

import hashlib
from aiges.sdk import WrapperBase, \
    StringParamField, \
    ImageBodyField, \
    StringBodyField
from aiges.utils.log import log

'''
定义请求类:
 params:  params 开头的属性代表最终HTTP协议中的功能参数parameters部分， 
          params Field支持 StringParamField，
          NumberParamField，BooleanParamField,IntegerParamField，每个字段均支持枚举
          params 属性多用于协议中的控制字段，请求body字段不属于params范畴

 input:    input字段多用与请求数据段，即body部分，当前支持 ImageBodyField, StringBodyField, 和AudioBodyField
'''


class UserRequest(object):
    params1 = StringParamField(key="p1", enums=["3", "eee"], value='3')
    params2 = StringParamField(key="p2", maxLength=44, required=True)
    params3 = StringParamField(key="p3", maxLength=44, required=False)

    input1 = ImageBodyField(key="data", path="test_data/test.png")
    input3 = ImageBodyField(key="data2", path="test_data/test.png")
    input2 = StringBodyField(key="switch", value="cc")


'''
定义响应类:
 accepts:  accepts代表响应中包含哪些字段, 以及数据类型

 input:    input字段多用与请求数据段，即body部分，当前支持 ImageBodyField, StringBodyField, 和AudioBodyField
'''


class UserResponse(object):
    accept1 = StringBodyField(key="boxes")
    accept2 = StringBodyField(key="boxes2")


'''
用户实现， 名称必须为Wrapper, 必须继承SDK中的 WrapperBase类
'''


class Wrapper(WrapperBase):
    serviceId = "mmocr"
    version = "backup.0"
    requestCls = UserRequest()
    responseCls = UserResponse()

    '''
    服务初始化
    @param config:
        插件初始化需要的一些配置，字典类型
        key: 配置名
        value: 配置的值
    @return
        ret: 错误码。无错误时返回0
    '''

    def wrapperInit(self, config: {}) -> int:
        print(config)
        print("Initializing ..")
        # 会话模式创建sessionManager
        self.session.init_wrapper_config(config)

        return 0

    '''
    非会话模式计算接口,对应oneShot请求,可能存在并发调用

    @param usrTag 句柄
    @param params 功能参数
    @param  reqData     请求数据实体字段
    @param  respData    返回结果实体,内存由底层服务层申请维护,通过execFree()接口释放
    @param psrIds 需要使用的个性化资源标识列表
    @param psrCnt 需要使用的个性化资源个数

    @return 接口错误码
        reqDat
        ret:错误码。无错误码时返回0
    '''

    def wrapperOnceExec(cls, params: {}, reqData: DataListCls) -> Response:
        log.info("got reqdata , %s" % reqData.list)
        #        print(type(reqData.list[0].data))
        #        print(type(reqData.list[0].data))
        #        print(reqData.list[0].len)
        for req in reqData.list:
            log.info("reqData key: %s , size is %d" % (req.key, len(req.data)))

        log.info("Testing reqData get: ")
        rg = reqData.get("data")
        log.info("get key: %s" % rg.key)
        log.info("get key: %d" % len(rg.data))

        # test not reqdata
        k = "dd"
        n = reqData.get(k)
        if not n:
            log.error("reqData not has this key %s" % k)

        log.warning("reqData bytes md5sum is %s" % hashlib.md5(reqData.list[0].data).hexdigest())
        log.info("I am infer logic...please inplement")

        r = Response()
        # return r.response_err(100)
        l = ResponseData()
        l.key = "ccc"
        l.status = 1
        d = open("test_data/test.png", "rb").read()
        l.len = len(d)
        l.data = d
        l.type = 0
        r.list = [l, l, l]
        return r

    '''
    服务逆初始化

    @return
        ret:错误码。无错误码时返回0
    '''

    def wrapperFini(cls) -> int:
        return 0

    def wrapperError(cls, ret: int) -> str:
        if ret == 100:
            return "user error defined here"
        return ""

    def wrapperTestFunc(cls, data: [], respData: []):
        r = Response()
        l = ResponseData()
        l.key = "ccc"
        l.status = 1
        d = open("pybind11/docs/pybind11-logo.png", "rb").read()
        l.len = len(d)
        l.data = d
        r.list = [l, l, l]

        print(r.list)
        print(444)
        return r

    def wrapperCreate(self, params: {}, sid: str) -> SessionCreateResponse:
        print(params)
        s = SessionCreateResponse()
        # 这里是取 handle
        handle = self.session.get_idle_handle()

        if not handle:
            s.error_code = -1
            s.handle = ""
            return  s
        _session = self.session.get_session(handle=handle)
        _session.setup_sid(sid)
        _session.setup_params(params)

        print(sid)
        s = SessionCreateResponse()
        s.handle = handle
        s.error_code = 0
        return s

    def wrapperWrite(self, handle:str, req: DataListCls, sid:str) -> int:
        print("handle",handle)
        print("sid:",sid)
        print("req:", req)
        for i in req.list:
            print(i.key)
            print(i.data)
        return 0

    def wrapperRead(self, handle:str, sid:str) -> Response:
        print("handle",handle)
        print("sid:",sid)
        _session = self.session.get_session(handle=handle)
        print("out.q", _session.out_q)
        r = Response()
        l = ResponseData()

        l.key = "ccc"
        l.status = 1
        d = b"cccccc"
        l.len = len(d)
        l.data = d
        r.list = [l]
        return r

if __name__ == '__main__':
    m = Wrapper()
    m.schema()
    m.run()



#!/usr/bin/env python
# coding:utf-8
""" 
@author: nivic ybyang7
@license: Apache Licence 
@file: call_wrapper.py
@time: 2022/06/15
@contact: ybyang7@iflytek.com
@site:  
@software: PyCharm 

# code is far away from bugs with the god animal protecting
    I love animals. They taste delicious.
              ┏┓      ┏┓
            ┏┛┻━━━┛┻┓
            ┃      ☃      ┃
            ┃  ┳┛  ┗┳  ┃
            ┃      ┻      ┃
            ┗━┓      ┏━┛
                ┃      ┗━━━┓
                ┃  神兽保佑    ┣┓
                ┃　永无BUG！   ┏┛
                ┗┓┓┏━┳┓┏┛
                  ┃┫┫  ┃┫┫
                  ┗┻┛  ┗┻┛ 
"""

#  Copyright (c) 2022. Lorem ipsum dolor sit amet, consectetur adipiscing elit.
#  Morbi non lorem porttitor neque feugiat blandit. Ut vitae ipsum eget quam lacinia accumsan.
#  Etiam sed turpis ac ipsum condimentum fringilla. Maecenas magna.
#  Proin dapibus sapien vel ante. Aliquam erat volutpat. Pellentesque sagittis ligula eget metus.
#  Vestibulum commodo. Ut rhoncus gravida arcu.

import argparse
import importlib
def call_wrapper(args):
    print(args)
    mod = importlib.import_module("wrapper")
    err_func =getattr(mod, "wrapperError")
    print(err_func(100))
    print(dir(mod))
    print("call done")


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-w', type=str,
                        default='wrapper.py',
                        help='wrapper.py file location')

    args = parser.parse_args()

    call_wrapper(args.w)

if __name__ == '__main__':
    main()


import argparse
from .create import *  # 初始化项目目录
from .config import *
from .package import *

import pkg_resources


def rootCmd():
    parser = argparse.ArgumentParser()

    subparsers = parser.add_subparsers(dest='action')

    # create project command
    create_parser = subparsers.add_parser('create')
    project_name = ""
    create_parser.add_argument('-n', '--name', help="初始化wrapper.py工程项目名称")
    create_parser.add_argument('-p', '--path', default="./", help="初始化工程位置")

    # version
    version = subparsers.add_parser('version')

    # package wrapper
    pack_parser = subparsers.add_parser('pack')
    pack_parser.add_argument('-l', '--location', help="工程目录路径", default="./")
    pack_parser.add_argument('-o', '--output', help="输出压缩路径", default="./")

    args = parser.parse_args()
    kwargs = vars(args)
    action = kwargs.pop('action')
    if action == "create":
        create_project(args)
    elif action == "version":
        print_version()
    elif action == "pack":
        pack(args)

    else:
        print("not supported for this command")


def registerSubcommand(parser, name):
    subparsers = parser.add_subparsers(dest=name)
    named_parser = subparsers.add_parser(name)
    return subparsers


def print_version():
    print(repr(pkg_resources.get_distribution('aiges')))



#!/usr/bin/env python
# coding:utf-8
""" 
@author: nivic ybyang7
@license: Apache Licence 
@file: stream
@time: 2022/09/20
@contact: ybyang7@iflytek.com
@site:  
@software: PyCharm 

# code is far away from bugs with the god animal protecting
    I love animals. They taste delicious.
              ┏┓      ┏┓
            ┏┛┻━━━┛┻┓
            ┃      ☃      ┃
            ┃  ┳┛  ┗┳  ┃
            ┃      ┻      ┃
            ┗━┓      ┏━┛
                ┃      ┗━━━┓
                ┃  神兽保佑    ┣┓
                ┃　永无BUG！   ┏┛
                ┗┓┓┏━┳┓┏┛
                  ┃┫┫  ┃┫┫
                  ┗┻┛  ┗┻┛ 
"""

#  Copyright (c) 2022. Lorem ipsum dolor sit amet, consectetur adipiscing elit.
#  Morbi non lorem porttitor neque feugiat blandit. Ut vitae ipsum eget quam lacinia accumsan.
#  Etiam sed turpis ac ipsum condimentum fringilla. Maecenas magna.
#  Proin dapibus sapien vel ante. Aliquam erat volutpat. Pellentesque sagittis ligula eget metus.
#  Vestibulum commodo. Ut rhoncus gravida arcu.
import threading
from threading import Lock
from multiprocessing import Manager, Process
from aiges.types import *
import time
import os
import queue

lock = Lock()


class SessionManager:
    """
    session管理器
    """

    def __init__(self, config={}, mode=THREAD_MODE, nums=0):
        self.initialized = False
        self.code = 0

        self.handle_pool: Pool  # handle池子，对应handle绑定的计算资源可以自行控制
        self.lock = threading.Lock()

    def init_wrapper_config(self, config={}):
        self.wrapperConfig = config

    def init_handle_pool(self, mode, nums, ReqThreadCls):
        if mode == THREAD_MODE:
            # 默认线程模式
            self.handle_pool = self.init_threads(ReqThreadCls, nums)
        elif mode == PROCESS_MODE:
            # 进程+线程模式当前不可用
            self.handle_pool = self.init_processes(ReqThreadCls, 2, 5)
        else:
            self.handle_pool = self.init_threads(ReqThreadCls, nums)
        self.initialized = True

    def init_threads(self, ReqThreadCls, nums=10):
        # 线程模式
        return ThreadPool(ReqThreadCls, 0, nums)

    def init_processes(self, ReqThreadCls, procs=2, threads_per_cpu=5):
        # 进程\线程模式
        return ProcessThreadPool(ReqThreadCls, procs, threads_per_cpu)

    def get_session(self, handle):
        # 通过handle名获取 session ,实际上取的是thread
        s = None
        if self.handle_pool.workers:
            s = self.handle_pool.get_thread(handle)
        if not s:
            pass
        return s

    def set_idle_session(self, handle):
        self.handle_pool.set_idle(handle)

    def get_idle_handle(self):
        # 获取一个空闲的session
        _session = None
        with lock:
            handle, _session = self.handle_pool.get_idle_thread_handle()
            if not _session:
                return
            return handle

    def reset_session(self, handle):
        _session = self.handle_pool.reset_thread(handle)

    def push_back(self, handle):
        # 标记这个handle对应线程为空闲
        self.set_idle_session(handle)
        self.reset_session(handle)
        pass


class Pool:
    """
    基类Pool
    """

    def __init__(self):
        # 维护 workers
        self.workers = {}

    def get_thread(self, handle):
        raise NotImplementedError

    def get_idle_thread_handle(self):
        "获取一个空闲的 线程handle"
        for h, s in self.workers.items():
            if s.idle:
                _session = s
                s.is_idle = False
                return h, s

    def reset_thread(self, handle):
        "重置一个线程状态，根据handle名"
        t = self.workers.get(handle)
        if not t:
            return
        t.reset()

    def set_idle(self, handle):
        t = self.workers.get(handle)
        if not t:
            return
        t.is_idle = True


class ThreadPool(Pool):
    """
    简易线程pool， 用于记录handle 和thread 绑定关系
    """

    def __init__(self, reqDataThreadCls, processs_no=0, nums=10, ):
        self.workers = {}
        for i in range(0, nums):
            handle = "Process-{}-Thread-{}".format(str(processs_no), str(i))
            self.workers[handle] = WorkerThread(handle)
            self.workers[handle].setStreamHandleThreadClass(reqDataThreadCls)
            self.workers[handle].start()

    def get_thread(self, handle):
        return self.workers.get(handle)


class StreamHandleThread(threading.Thread):
    """
    流式处理worker线程基类:

    """

    def __init__(self, session_thread, in_q, out_q):
        threading.Thread.__init__(self)
        self.session_thread = session_thread  # 为worker线程的父线程
        self.in_q = in_q
        self.out_q = out_q

    def init_model(self, *args, **kwarg):
        raise NotImplementedError("implmenthis")

    def run(self):
        # 此方法需要子类重写
        self.init_model(self.session_thread.handle)
        while True:
            time.sleep(100)
            # req = self.in_q.get()
            # if req.list[0].status == 2:
            #    self.session_thread.reset()


class WorkerProcess(Process):
    """
    TODO: Depreciated 进程线程模式当前不可用
    """

    def __init__(self, workers, reqDataThreadCls, process_no=0, threads_per_cpu=2):
        Process.__init__(self)
        self.workers = workers
        self.process_no = process_no
        self.threads_per_cpu = threads_per_cpu
        self.reqDataThreadCls = reqDataThreadCls
        self.process_id = None

    # 重写run方法
    def run(self):
        self.process_id = os.getpid()
        for i in range(0, self.threads_per_cpu):
            handle = "Process-{}-Thread-{}".format(str(os.getpid()), str(i))
            print(handle + "start")
            self.workers[handle] = WorkerThread(handle)
            self.workers[handle].setStreamHandleThreadClass(self.reqDataThreadCls)
            self.workers[handle].start()


class ProcessThreadPool(Pool):
    """
    TODO: Depreciated 进程线程模式当前不可用
    """

    def __init__(self, reqWorkerHandleCls, procs=2, threads_per_cpu=5):
        self.workers = {}
        self.process = {}
        for i in range(0, procs):
            # 初始化进程
            p = WorkerProcess(self.workers, reqWorkerHandleCls, str(i), threads_per_cpu)
            p.start()
            self.process[p.process_id] = p

    def get_thread(self, handle):
        return self.workers.get(handle)


class WorkerThread(threading.Thread):
    """
    Worker线程, 负责记录每次会话的上下文数据
    """

    # 继承父类threading.Thread
    def __init__(self, handle):
        threading.Thread.__init__(self)
        self.handle = handle  # handle标识，此处指的的是线程handle标识


        self.sid = ""  # 会话ID,由外部请求输入

        self.in_q = queue.Queue()  # 定义输入工作线程队列
        self.out_q = queue.Queue()  # 异步callback获取结果时，此队列无用，仅同步取结果时有用
        self.is_idle = True  # 标明当前线程是否空闲，空闲则可以被获取
        self.params = {}  # 记录某次会话的用户功能参数

        self.StreamHandleThreadClass = StreamHandleThread  # 默认的流式处理线程类，实际使用时，请继承此类并自定义run方法

        self.callback_fn = None  # 该回调为，运行时，c++传入的会话callback，函数形式为 callback(response:Response , sid:string),
        # 用于回调数据返回

    def init_model(self, *args, **kwargs):
        raise NotImplementedError("Please implement this method...")

    @property
    def idle(self):
        return self.is_idle

    def setup_sid(self, sid):
        # 设置会话ID
        self.sid = sid

    def setup_params(self, params):
        # 设置会话参数
        self.params = params
        self.is_idle = False

    def setup_callback_fn(self, callback):
        # 设置会话callback，用于返回响应数据，函数形式为 callback(response:Response , sid:string),
        self.callback_fn = callback

    def reset(self):
        self.sid = ""
        self.params = {}
        with self.in_q.mutex:
            self.in_q.queue.clear()
        with self.out_q.mutex:
            self.out_q.queue.clear()
        self.is_idle = True

    def setStreamHandleThreadClass(self, cls):
        if issubclass(self.StreamHandleThreadClass, StreamHandleThread):
            print("handle thread set up ok...")
        self.StreamHandleThreadClass = cls

    def run(self):
        pr = self.StreamHandleThreadClass(self, self.in_q, self.out_q)
        pr.start()
        pr.join()



import sys

if not hasattr(sys, 'argv'):
    sys.argv = ['']

'''
服务初始化
@param config:
    插件初始化需要的一些配置，字典类型
    key: 配置名
    value: 配置的值
@return
    ret: 错误码。无错误时返回0
'''


def wrapperInit(config: {}) -> int:
    return 0

'''
服务逆初始化

@return
    ret:错误码。无错误码时返回0
'''

def wrapperFini() -> int:
    return 0

'''
非会话模式计算接口,对应oneShot请求,可能存在并发调用

@param usrTag 句柄
#param params 功能参数
@param  reqData     写入数据实体
@param  respData    返回结果实体,内存由底层服务层申请维护,通过execFree()接口释放
@param psrIds 需要使用的个性化资源标识列表
@param psrCnt 需要使用的个性化资源个数

@return 接口错误码
    reqDat
    ret:错误码。无错误码时返回0
'''


def wrapperOnceExec(usrTag: str, params: {}, reqData: [], respData: [], psrIds: [], psrCnt: int) -> int:
    print("hello world")
    print(usrTag)
    print(params)
    print(reqData)
    print(psrIds)
    print(psrCnt)
    return 100


def wrapperCreate(usrTag: str, params: [], psrIds: [], psrCnt: int) -> str:
    return ""


def wrapperWrite(handle: str, datas: []) -> int:
    return 0


def wrapperRead(handle: str) -> []:
    return []


def wrapperDestroy(handle: str) -> int:
    return 0


def wrapperError(ret: int) -> str:
    if ret == 100:
        return "this is a tese error return"
    return ""



#!/usr/bin/env python
# coding:utf-8
""" 
@author: nivic ybyang7
@license: Apache Licence 
@file: flask_utils
@time: 2022/08/06
@contact: ybyang7@iflytek.com
@site:  
@software: PyCharm 

# code is far away from bugs with the god animal protecting
    I love animals. They taste delicious.
              ┏┓      ┏┓
            ┏┛┻━━━┛┻┓
            ┃      ☃      ┃
            ┃  ┳┛  ┗┳  ┃
            ┃      ┻      ┃
            ┗━┓      ┏━┛
                ┃      ┗━━━┓
                ┃  神兽保佑    ┣┓
                ┃　永无BUG！   ┏┛
                ┗┓┓┏━┳┓┏┛
                  ┃┫┫  ┃┫┫
                  ┗┻┛  ┗┻┛ 
"""

#  Copyright (c) 2022. Lorem ipsum dolor sit amet, consectetur adipiscing elit.
#  Morbi non lorem porttitor neque feugiat blandit. Ut vitae ipsum eget quam lacinia accumsan.
#  Etiam sed turpis ac ipsum condimentum fringilla. Maecenas magna.
#  Proin dapibus sapien vel ante. Aliquam erat volutpat. Pellentesque sagittis ligula eget metus.
#  Vestibulum commodo. Ut rhoncus gravida arcu.
import os


def getenv_as_bool(*env_vars, default=False):
    """
    Read environment variable, parsing it to a boolean.
    """

    val = getenv(*env_vars)

    if val is None:
        return default

    return val.lower() in ["1", "true", "t"]

def getenv(*env_vars, default=None):
    """
    Overload of os.getenv() to allow falling back through multiple environment
    variables. The environment variables will be checked sequentially until one
    of them is found.

    Parameters
    ------
    *env_vars
        Variadic list of environment variable names to check.
    default
        Default value to return if none of the environment variables exist.

    Returns
    ------
        Value of the first environment variable set or default.
    """
    for env_var in env_vars:
        if env_var in os.environ:
            return os.environ.get(env_var)

    return default


class AigesMicroserviceException(Exception):
    status_code = 400

    def __init__(
        self, message, status_code=None, payload=None, reason="MICROSERVICE_BAD_DATA"
    ):
        Exception.__init__(self)
        self.message = message
        if status_code is not None:
            self.status_code = status_code
        self.payload = payload
        self.reason = reason

    def to_dict(self):
        rv = {
            "status": {
                "status": 1,
                "info": self.message,
                "code": -1,
                "reason": self.reason,
            }
        }
        return rv



#!/usr/bin/env python
# coding:utf-8
"""
@license: Apache License2
@file: wrapper.py
@time: 2022-08-03 09:41:20.346690
@project: mmocr_v2
@project: ./
"""

import sys
import hashlib
import time

try:
    from aiges_embed import ResponseData, Response, DataListNode, DataListCls, SessionCreateResponse, callback  # c++
except:
    from aiges.dto import Response, ResponseData, DataListNode, DataListCls

from aiges.sdk import WrapperBase, \
    StringParamField, \
    ImageBodyField, \
    StringBodyField

from aiges.stream import StreamHandleThread

from aiges.utils.log import log
from aiges.types import *

########
# 请在此区域导入您的依赖库

# Todo
# for example: import numpy
import numpy as np
from PIL import Image
import io
from mmocr.utils.ocr import MMOCR
import json

########


"""
定义请求类:
 params:  params 开头的属性代表最终HTTP协议中的功能参数parameters部分，
          params Field支持 StringParamField，
          NumberParamField，BooleanParamField,IntegerParamField，每个字段均支持枚举
          params 属性多用于协议中的控制字段，请求body字段不属于params范畴

 input:    input字段多用与请求数据段，即body部分，当前支持 ImageBodyField, StringBodyField, 和AudioBodyField
"""


class UserRequest(object):
    # StringParamField多用于控制参数
    # 指明 enums, maxLength, required有助于自动根据要求配置协议schema
    # params1 = StringParamField(key="p1", enums=["3", "eee"], value='3')
    # params2 = StringParamField(key="p2", maxLength=44, required=True)
    # params3 = StringParamField(key="p3", maxLength=44, required=False)

    # imagebodyfield 指明path，有助于本地调试wrapper.py
    input1 = ImageBodyField(key="data", path="demo/demo_text_det.jpg")
    # input3 = ImageBodyField(key="data2", path="test_data/test.png")
    # stringbodyfiled 指明 value，用于本地调试时的测试值
    # input2 = StringBodyField(key="switch", value="ctrl")


"""
定义响应类:
 accepts:  accepts代表响应中包含哪些字段, 以及数据类型

 input:    input字段多用与请求数据段，即body部分，当前支持 ImageBodyField, StringBodyField, 和AudioBodyField
"""


class UserResponse(object):
    # 此类定义响应返回数据段，请务必指明对应key
    # 支持 ImageBodyField， AudioBodyField,  StringBodyField
    # 如果响应是json， 请使用StringBodyField
    accept1 = StringBodyField(key="boxes")


class Wrapper(WrapperBase):
    """
    用户实现， 名称必须为Wrapper, 必须继承SDK中的 WrapperBase类
    """
    serviceId = "mmocr"
    version = "backup.0"
    requestCls = UserRequest()
    responseCls = UserResponse()

    mode = None
    session_total = 0  # 单进程总会话数量控制，过大会影响效率，请合理控制

    def wrapperInit(self, config: {}) -> int:
        """
        服务初始化
        @param config:
            插件初始化需要的一些配置，字典类型
            key: 配置名
            value: 配置的值
        @return
            ret: 错误码。无错误时返回0
        """
        log.info(config)
        Wrapper.model = MMOCR()
        log.info("Initializing ...")
        Wrapper.session_total = config.get("common.lic", 10)
        self.session.init_wrapper_config(config)
        self.session.init_handle_pool("thread", 10, MyReqDataThread)
        return 0

    def wrapperOnceExec(cls, params: {}, reqData: DataListCls) -> Response:
        """
        非会话模式计算接口,对应oneShot请求,可能存在并发调用
        @param params 功能参数
        @param  reqData     请求数据实体字段 DataListCls,可通过 aiges.dto.DataListCls查看
        @return
            响应必须返回 Response类，非Response类将会引起未知错误
        """
        return None

    def wrapperFini(cls) -> int:
        """
        服务逆初始化

        @return
            ret:错误码。无错误码时返回0
        """
        log.info("fini success")
        return 0

    def wrapperError(cls, ret: int) -> str:

        if ret == 10013:
            return "reqData is empty"
        elif ret == 10001:
            return "load onnx model failed"
        else:
            return "other error code"

    def wrapperCreate(self, params: {}, sid: str) -> SessionCreateResponse:
        """
        非会话模式计算接口,对应oneShot请求,可能存在并发调用
        @param ret wrapperOnceExec返回的response中的error_code 将会被自动传入本函数并通过http响应返回给最终用户
        @return
            SessionCreateResponse类, 如果返回不是该类会报错
        """
        s = SessionCreateResponse()
        # 这里是取 handle
        handle = self.session.get_idle_handle()
        if not handle:
            s.error_code = -1
            s.handle = ""
            return s

        _session = self.session.get_session(handle=handle)
        if _session == None:
            log.info("can't create this handle:" % handle)
            return -1
        _session.setup_sid(sid)
        _session.setup_params(params)
        _session.setup_callback_fn(callback)

        print(sid)
        s = SessionCreateResponse()
        s.handle = handle
        s.error_code = 0
        return s

    def wrapperWrite(self, handle: str, req: DataListCls, sid: str) -> int:
        """
        会话模式下: 上行数据写入接口

        :param handle: 会话handle 字符串
        :param req:  请求数据结构
        :param sid:  请求会话ID
        :return:
        """
        _session = self.session.get_session(handle=handle)
        if _session == None:
            log.info("can't get this handle:" % handle)
            return -1
        _session.in_q.put(req)
        return 0

    def wrapperRead(self, handle: str, sid: str) -> Response:
        """
        会话模式: 当前此接口在会话模式且异步取结果时下不会被调用！！！！！返回数据由callback返回
        同步取结果模式时，下行数据返回接口
                  如果为异步返回结果时，需要设置加载器为asyncMode=true [当前默认此模式],
        :param handle: 请求数据结构
        :param sid: 请求会话ID
        :return: Response类
        """
        _session = self.session.get_session(handle=handle)
        r = Response()
        l = ResponseData()
        if _session.out_q.empty():
            l.status = 1
            l.len = 0
            l.data = b''
            l.key = "boxes"
            r.list = [l]
            return r
        rs = _session.out_q.get()
        if rs.list[0].status == DataEnd:
            print("########Done")
            _session.reset()
        if not isinstance(rs, Response):
            raise Exception("check response")

        return rs

    def wrapperTestFunc(cls, data: [], respData: []):
        """
        此函数保留测试用，不可删除
        :param data:
        :param respData:
        :return:
        """

        r = Response()
        l = ResponseData()
        l.key = "boxes"
        l.status = 1
        d = open("pybind11/docs/pybind11-logo.png", "rb").read()
        l.len = len(d)
        l.data = d
        r.list = [l, l, l]

        print(r.list)
        print(444)
        return r


class MyReqDataThread(StreamHandleThread):
    """
    流式示例 thread，
    """

    def __init__(self, session_thread, in_q, out_q):
        super().__init__(session_thread, in_q, out_q)

    def run(self):
        while True:
            req = self.in_q.get()
            print(self.session_thread.params)
            self.infer(req)

    def infer(self, req: DataListCls):
        print("data list size,", len(req.list))
        print(req.list[0].len)
        print(req.list[0].status)
        r = Response()
        l = ResponseData()
        l.key = "boxes"
        l.status = req.list[0].status
        d = open("/home/mmocr/resources/mmocr-logo.png", "rb").read()
        # d = b"ccccccc"
        l.len = len(d)
        l.data = d
        r.list = [l]
        self.session_thread.callback_fn(r, self.session_thread.sid)
        if l.status == DataEnd:
            self.session_thread.reset()  # 最后一帧数据后 释放会话
        # self.out_q.put(r)

    def init_model(self, *args, **kwargs):
        model = None
        return model


if __name__ == '__main__':
    m = Wrapper()
    # m.schema()
    m.run()



#!/usr/bin/env python
# coding:utf-8
""" 
@author: nivic ybyang7
@license: Apache Licence 
@file: rest
@time: 2022/08/06
@contact: ybyang7@iflytek.com
@site:  
@software: PyCharm 

# code is far away from bugs with the god animal protecting
    I love animals. They taste delicious.
              ┏┓      ┏┓
            ┏┛┻━━━┛┻┓
            ┃      ☃      ┃
            ┃  ┳┛  ┗┳  ┃
            ┃      ┻      ┃
            ┗━┓      ┏━┛
                ┃      ┗━━━┓
                ┃  神兽保佑    ┣┓
                ┃　永无BUG！   ┏┛
                ┗┓┓┏━┳┓┏┛
                  ┃┫┫  ┃┫┫
                  ┗┻┛  ┗┻┛ 
"""

#  Copyright (c) 2022. Lorem ipsum dolor sit amet, consectetur adipiscing elit.
#  Morbi non lorem porttitor neque feugiat blandit. Ut vitae ipsum eget quam lacinia accumsan.
#  Etiam sed turpis ac ipsum condimentum fringilla. Maecenas magna.
#  Proin dapibus sapien vel ante. Aliquam erat volutpat. Pellentesque sagittis ligula eget metus.
#  Vestibulum commodo. Ut rhoncus gravida arcu.

import logging
from flask import Flask, Response, request, send_from_directory
from flask_cors import CORS
from aiges.utils.flask_utils import *


logger = logging.getLogger(__name__)


def get_rest_microservice(user_model, seldon_metrics):
    app = Flask(__name__, static_url_path="")
    CORS(app)

    _set_flask_app_configs(app)

    # dict representing the validated model metadata
    # None value will represent a validation error
    metadata_data = seldon_core.seldon_methods.init_metadata(user_model)

    if hasattr(user_model, "model_error_handler"):
        logger.info("Registering the custom error handler...")
        app.register_blueprint(user_model.model_error_handler)

    @app.errorhandler(Exception)
    def handle_generic_exception(e):
        error = SeldonMicroserviceException(
            message=str(e), status_code=500, reason="MICROSERVICE_INTERNAL_ERROR"
        )
        response = jsonify(error.to_dict())
        logger.error("%s", error.to_dict())
        response.status_code = error.status_code
        return response

    @app.errorhandler(SeldonMicroserviceException)
    def handle_invalid_usage(error):
        response = jsonify(error.to_dict())
        logger.error("%s", error.to_dict())
        response.status_code = error.status_code
        return response

    @app.route("/seldon.json", methods=["GET"])
    def openAPI():
        return send_from_directory("", "openapi/seldon.json")

    @app.route("/predict", methods=["GET", "POST"])
    @app.route("/api/v1.0/predictions", methods=["POST"])
    @app.route("/api/v0.1/predictions", methods=["POST"])
    def Predict():
        requestJson = get_request(skip_decoding=PAYLOAD_PASSTHROUGH)
        logger.debug("REST Request: %s", request)
        response = seldon_core.seldon_methods.predict(
            user_model, requestJson, seldon_metrics
        )

        json_response = jsonify(response, skip_encoding=PAYLOAD_PASSTHROUGH)
        if (
            isinstance(response, dict)
            and "status" in response
            and "code" in response["status"]
        ):
            json_response.status_code = response["status"]["code"]

        logger.debug("REST Response: %s", response)
        return json_response

    @app.route("/send-feedback", methods=["GET", "POST"])
    @app.route("/api/v1.0/feedback", methods=["POST"])
    @app.route("/api/v0.1/feedback", methods=["POST"])
    def SendFeedback():
        requestJson = get_request()
        logger.debug("REST Request: %s", request)
        requestProto = json_to_feedback(requestJson)
        logger.debug("Proto Request: %s", requestProto)
        responseProto = seldon_core.seldon_methods.send_feedback(
            user_model, requestProto, PRED_UNIT_ID, seldon_metrics
        )
        jsonDict = seldon_message_to_json(responseProto)
        return jsonify(jsonDict)

    @app.route("/transform-input", methods=["GET", "POST"])
    def TransformInput():
        requestJson = get_request()
        logger.debug("REST Request: %s", request)
        response = seldon_core.seldon_methods.transform_input(
            user_model, requestJson, seldon_metrics
        )
        logger.debug("REST Response: %s", response)
        return jsonify(response)

    @app.route("/transform-output", methods=["GET", "POST"])
    def TransformOutput():
        requestJson = get_request()
        logger.debug("REST Request: %s", request)
        response = seldon_core.seldon_methods.transform_output(
            user_model, requestJson, seldon_metrics
        )
        logger.debug("REST Response: %s", response)
        return jsonify(response)

    @app.route("/route", methods=["GET", "POST"])
    def Route():
        requestJson = get_request()
        logger.debug("REST Request: %s", request)
        response = seldon_core.seldon_methods.route(
            user_model, requestJson, seldon_metrics
        )
        logger.debug("REST Response: %s", response)
        return jsonify(response)

    @app.route("/aggregate", methods=["GET", "POST"])
    def Aggregate():
        requestJson = get_request()
        logger.debug("REST Request: %s", request)
        response = seldon_core.seldon_methods.aggregate(
            user_model, requestJson, seldon_metrics
        )
        logger.debug("REST Response: %s", response)
        return jsonify(response)

    @app.route("/health/ping", methods=["GET"])
    def HealthPing():
        """
        Lightweight endpoint to check the liveness of the REST endpoint
        """
        return "pong"

    @app.route("/health/status", methods=["GET"])
    @app.route("/api/v1.0/health/status", methods=["GET"])
    def HealthStatus():
        logger.debug("REST Health Status Request")
        response = seldon_core.seldon_methods.health_status(user_model, seldon_metrics)
        logger.debug("REST Health Status Response: %s", response)
        return jsonify(response)

    @app.route("/metadata", methods=["GET"])
    @app.route("/api/v1.0/metadata", methods=["GET"])
    def Metadata():
        if metadata_data is None:
            # None value represents validation error in current implementation
            # if user_model would not define init_metadata than metadata_data
            # would just contain a default values
            raise SeldonMicroserviceException(
                "Model metadata unavailable",
                status_code=500,
                reason="MICROSERVICE_BAD_METADATA",
            )
        logger.debug("REST Metadata Request")
        logger.debug("REST Metadata Response: %s", metadata_data)
        return jsonify(metadata_data)

    return app


def _set_flask_app_configs(app):
    """
    Set the configs for the flask app based on environment variables
    See https://flask.palletsprojects.com/config/#builtin-configuration-values
    :param app:
    :return:
    """
    FLASK_CONFIG_IDENTIFIER = "FLASK_"
    FLASK_CONFIGS_ALLOWED = [
        "DEBUG",
        "EXPLAIN_TEMPLATE_LOADING",
        "JSONIFY_PRETTYPRINT_REGULAR",
        "JSON_SORT_KEYS",
        "PROPAGATE_EXCEPTIONS",
        "PRESERVE_CONTEXT_ON_EXCEPTION",
        "SESSION_COOKIE_HTTPONLY",
        "SESSION_COOKIE_SECURE",
        "SESSION_REFRESH_EACH_REQUEST",
        "TEMPLATES_AUTO_RELOAD",
        "TESTING",
        "TRAP_HTTP_EXCEPTIONS",
        "TRAP_BAD_REQUEST_ERRORS",
        "USE_X_SENDFILE",
    ]

    for flask_config in FLASK_CONFIGS_ALLOWED:
        flask_config_value = getenv_as_bool(
            f"{FLASK_CONFIG_IDENTIFIER}{flask_config}", default=None
        )
        if flask_config_value is None:
            continue
        app.config[flask_config] = flask_config_value
    logger.info(f"App Config:  {app.config}")




#!/usr/bin/env python
# coding:utf-8
""" 
@author: nivic ybyang7
@license: Apache Licence 
@file: aiges_methods.py
@time: 2022/08/06
@contact: ybyang7@iflytek.com
@site:  
@software: PyCharm 

# code is far away from bugs with the god animal protecting
    I love animals. They taste delicious.
              ┏┓      ┏┓
            ┏┛┻━━━┛┻┓
            ┃      ☃      ┃
            ┃  ┳┛  ┗┳  ┃
            ┃      ┻      ┃
            ┗━┓      ┏━┛
                ┃      ┗━━━┓
                ┃  神兽保佑    ┣┓
                ┃　永无BUG！   ┏┛
                ┗┓┓┏━┳┓┏┛
                  ┃┫┫  ┃┫┫
                  ┗┻┛  ┗┻┛ 
"""

#  Copyright (c) 2022. Lorem ipsum dolor sit amet, consectetur adipiscing elit.
#  Morbi non lorem porttitor neque feugiat blandit. Ut vitae ipsum eget quam lacinia accumsan.
#  Etiam sed turpis ac ipsum condimentum fringilla. Maecenas magna.
#  Proin dapibus sapien vel ante. Aliquam erat volutpat. Pellentesque sagittis ligula eget metus.
#  Vestibulum commodo. Ut rhoncus gravida arcu.
import logging
import os
from typing import Any, Dict, List, Tuple, Union
from aiges.utils.flask_utils import *
import yaml

logger = logging.getLogger(__name__)



def init_metadata(user_model: Any) -> Dict:
    """
    Call the user model to get the model init_metadata

    Parameters
    ----------
    user_model
        User defined class instance

    Returns
    -------
        Validated model metadata
    """
    # meta_user: load metadata defined in the user_model instance
    if hasattr(user_model, "init_metadata"):
        try:
            meta_user = user_model.init_metadata()
        except AigesMicroserviceException:
            meta_user = {}
            pass
    else:
        meta_user = {}

    if not isinstance(meta_user, dict):
        logger.error("init_metadata must return dict")
        meta_user = {}

    # meta_env: load metadata from environmental variable
    try:
        meta_env = yaml.safe_load(os.environ.get("MODEL_METADATA", "{}"))
    except yaml.YAMLError as e:
        logger.error(f"Reading metadata from MODEL_METADATA env variable failed: {e}")
        meta_env = {}

    meta = {**meta_user, **meta_env}

    try:
        return validate_model_metadata(meta)
    except SeldonInvalidMetadataError as e:
        logger.error(f"Metadata validation error\n{e}")
        logger.error(f"Failed to validate metadata {meta}")
        return None



#!/usr/bin/env python
# coding:utf-8
""" 
@author: nivic ybyang7
@license: Apache Licence 
@file: create.py
@time: 2022/07/26
@contact: ybyang7@iflytek.com
@site:  
@software: PyCharm 

# code is far away from bugs with the god animal protecting
    I love animals. They taste delicious.
              ┏┓      ┏┓
            ┏┛┻━━━┛┻┓
            ┃      ☃      ┃
            ┃  ┳┛  ┗┳  ┃
            ┃      ┻      ┃
            ┗━┓      ┏━┛
                ┃      ┗━━━┓
                ┃  神兽保佑    ┣┓
                ┃　永无BUG！   ┏┛
                ┗┓┓┏━┳┓┏┛
                  ┃┫┫  ┃┫┫
                  ┗┻┛  ┗┻┛ 
"""
import datetime

import os
import shutil
from typing import Any
from jinja2 import Environment
import aiges
from aiges.utils.log import log

WRAPPER_DIR = "wrapper"
TPL_DIR = os.path.join(os.path.dirname(aiges.__file__), "tpls")

template_env: Any = Environment(
    extensions=["jinja2.ext.do", "jinja2.ext.loopcontrols"],
    trim_blocks=True,
    lstrip_blocks=True,
)


def _load_template(tpl):
    if not os.path.exists(tpl):
        raise FileNotFoundError("not found %s" % tpl)
    log.info("load success %s" % tpl)
    return template_env.from_string(open(tpl, "r").read())


def load_readme():
    tpl = os.path.join(TPL_DIR, "Readme.md.j2")
    return _load_template(tpl)


def load_requirements():
    tpl = os.path.join(TPL_DIR, "requirements.txt.j2")
    return _load_template(tpl)


def load_dockerfile():
    tpl = os.path.join(TPL_DIR, "Dockerfile.j2")
    return _load_template(tpl)


def load_wrapper():
    tpl = os.path.join(TPL_DIR, "wrapper.py.j2")
    return _load_template(tpl)


def create_project(args):
    project_name = args.name
    project_path = args.path
    log.info("creating project: %s" % project_name)
    # todo
    # 1. 创建 project目录
    # 2. 渲染readme.md
    # 3. 渲染 wrapper.py
    # 4. 渲染 dockerfile
    # 5. 渲染 requirments.txt
    # 假设先实现1个readme,以下是demo， 请自己优化
    initialize_project_dir(project_name, project_path)


def initialize_project_dir(project_name, project_path):
    """

    :param project_name:  待初始化项目名称
    :param project_path:  待初始化项目地址
    :return:
    """
    # todo
    # 1. 创建 project_name的目录
    # 2. di
    abs = os.path.join(project_path, project_name)
    abs_wrapper_dir = os.path.join(abs, WRAPPER_DIR)
    os.makedirs(abs, exist_ok=True)
    os.makedirs(abs_wrapper_dir, exist_ok=True)

    # 创建示例 wrapper.py
    log.info("Templates file dir: %s" % TPL_DIR)
    tpls = os.listdir(TPL_DIR)
    log.info("Templates: %s" % str(tpls))
    wrapper_t = load_wrapper()
    wrapper_vars = {
        "datetime": datetime.datetime.now(),
        "project_name": project_name,
        "project_path": project_path
    }
    wfile = os.path.join(abs_wrapper_dir, "wrapper.py")
    log.info("Generating: %s" % wfile)
    with open((wfile), "wb") as wf:
        wf.write(wrapper_t.render(vars=wrapper_vars).encode('utf-8'))
        wf.close()
    log.info("Generated: %s >>> Done" % wfile)

    # 创建 Readme.md
    readme_t = load_readme()
    readme_vars = {
        "project_name": project_name,
        "project_path": project_path,
        "test": 'for test',

    }
    readme_file = os.path.join(abs, "README.md")
    log.info("Generating: %s" % readme_file)
    with open(readme_file, 'wb') as rf:
        rf.write(readme_t.render(vars=readme_vars).encode('utf-8'))
        rf.close()
    log.info("Generated: %s >>> Done" % readme_file)

    # 创建 requirements.txt
    require_t = load_requirements()
    require_vars = {}
    require_file = os.path.join(abs, "requirements.txt")
    log.info("Generating: %s" % require_file)
    with open(require_file, 'wb') as rf:
        rf.write(require_t.render(vars=require_vars).encode('utf-8'))
        rf.close()
    log.info("Generated: %s >>> Done" % require_file)

    # 创建 Dockerfile
    dockerfile_t = load_dockerfile()
    dockerfile_vars = {}
    dockerfile_file = os.path.join(abs, "Dockerfile")
    log.info("Generating: %s" % dockerfile_file)
    with open(dockerfile_file, 'wb') as rf:
        rf.write(dockerfile_t.render(vars=dockerfile_vars).encode('utf-8'))
        rf.close()
    log.info("Generated: %s >>> Done" % dockerfile_file)

    # 创建测试数据目录
    test_data = os.path.join(abs_wrapper_dir, "test_data")
    os.makedirs(test_data,exist_ok=True)
    test_png = os.path.join(abs_wrapper_dir, "test_data","test.png")
    shutil.copy(os.path.join(os.path.dirname(aiges.__file__),"test_data","test.png"), test_png)



#!/usr/bin/env python
# coding:utf-8
""" 
@author: nivic ybyang7
@license: Apache Licence 
@file: create.py
@time: 2022/07/26
@contact: ybyang7@iflytek.com
@site:  
@software: PyCharm
管理配置中心的
"""

def create_config(name):
    print("creating project")


# coding: utf-8

## const default value
import queue

import time
import base64

#
try:
    from aiges_embed import ResponseData, Response, DataListNode, DataListCls, SessionCreateResponse
except:
    from aiges.dto import Response, ResponseData, DataListNode, DataListCls, DataAudio, SessionCreateResponse

from jinja2 import Template
import json
import os
from aiges.utils.log import log
import threading
from pprint import pprint
from aiges.types import *
from aiges.stream import *
import queue
from threading import Lock
from multiprocessing import Process

SUB = "ase"
CALL = "atmos"
CALL_TYPE = 0
HOSTS = [
    "api.xf-yun.com"
]
ROUTEKEY = []

type_map = {
    "text": STRING,
    "audio": AUDIO,
    "image": IMAGE,
    "video": VIDEO
}

tpl = '''
{
    "meta":{
        "serviceId":"{{serviceId}}",
        "version":"{{version}}",
        "route":"{{route}}",
        "sub":"{{sub}}",
        "call":"{{call}}",
        "call_type":{{call_type}},
        "hosts":"{{ hosts}}",
        "service": {{service}},
        "routeKey":[
		],
		"{{serviceId}}":{
            "input":{{inputs_fields}},
            "accept":{{accepts_fields}}
        }
    },
    "schemainput":{
        "type":"object",
        "properties":{
            "header":{
                "type":"object",
                "properties":{
                    "appid":{
                        "type":"string",
                        "maxLength":50
                    },
                    "uid":{
                        "type":"string",
                        "maxLength":50
                    },
                    "device_id":{
                        "type":"string",
                        "maxLength":50
                    },
                    "device.imei":{
                        "type":"string",
                        "maxLength":50
                    },
                    "device.imsi":{
                        "type":"string",
                        "maxLength":50
                    },
                    "device.mac":{
                        "type":"string",
                        "maxLength":50
                    },
                    "device.other":{
                        "type":"string",
                        "maxLength":50
                    },
                    "net.type":{
                        "type":"string",
                        "maxLength":50
                    },
                    "net.isp":{
                        "type":"string",
                        "maxLength":50
                    },
                    "app.ver":{
                        "type":"string",
                        "maxLength":50
                    }
                },
                "required":[
                    "appid"
                ]
            },
            "parameter":{
                "type":"object",
                "properties":{
                    "{{serviceId}}":{
                        "type":"object",
                        "properties":{{parameters}},
                        "required": {{required_paramters}}
                    }
                }
            },
            "payload":{
                "type":"object",
                "properties": {{inputs_payload}}
            }
        }
    },
    "schemaoutput":{
        "type":"object",
        "properties":{
            "payload":{
                "type":"object",
                "properties": {{accepts_payloads}}
            }
        }
    }
}
'''


class Field(object):
    def __init__(self, key, data_type):
        self.key = key
        self.data_type = data_type

    def __str__(self):
        return '<%s:%s>' % (self.__class__.__name__, self.key)

    def _check_path(self, path):
        if not os.path.exists(path):
            e = FileNotFoundError
            log.warn(e)
        return path

    def _encode(self, dataTye, encoding, binary):
        print("Not implement here , just give back")
        return binary

    @property
    def test_value(self):
        return NotImplementedError


class ParamField(Field):
    pass


class PayloadField(Field):
    pass


class JsonBodyField(PayloadField):
    def __init__(self, key, value="", need_base64=False):
        super(JsonBodyField, self).__init__(key, STRING)
        self.value = value
        self.data_type = STRING
        self.need_base64 = need_base64
        self.key = key

    @property
    def test_value(self):
        if not self.need_base64:
            return self.value
        else:
            return base64.b64encode(self.value)


class StringBodyField(PayloadField):
    def __init__(self, key, value="", need_base64=False):
        super(StringBodyField, self).__init__(key, STRING)
        self.value = value
        self.data_type = STRING
        self.need_base64 = need_base64
        self.key = key

    @property
    def test_value(self):
        if not self.need_base64:
            return self.value
        else:
            return base64.b64encode(self.value)


class AudioBodyField(PayloadField):
    def __init__(self, key, path=""):
        super(AudioBodyField, self).__init__(key, AUDIO)
        self.data_type = AUDIO
        self.path = self._check_path(path)
        self.key = key

    @property
    def test_value(self):
        with open(self.path, "rb") as f:
            return f.read()


class ImageBodyField(PayloadField):
    def __init__(self, key, path="", need_base64=False):
        super(ImageBodyField, self).__init__(key, IMAGE)
        self.need_base64 = need_base64
        self.data_type = IMAGE
        self.path = self._check_path(path)
        self.key = key

    @property
    def test_value(self):
        with open(self.path, "rb") as f:
            content = f.read()
            if not self.need_base64:
                return content
            else:
                return base64.b64encode(content)


class StringParamField(ParamField):
    def __init__(self, key, minLength=0, maxLength=50, enums=[], required=False, value=""):
        self.key = key
        self.data_type = "string"
        self.max_length = maxLength
        self.min_length = minLength
        self.value = value
        self.enums = enums
        self.required = required

    def _schema(self):
        # todo enhance
        sc = {self.key: {"type": self.data_type}}
        if self.enums:
            sc[self.key]['enum'] = self.enums
        else:
            sc[self.key]['maxLength'] = self.max_length
        return sc

    def _to_dict(self):
        if not self.enums:
            return {self.key: {
                "type": self.data_type,
                "minLength": self.min_length,
                "maxLength": self.max_length
            }}
        return {self.key: {
            "type": self.data_type,
            "enum": self.enums
        }}

    @property
    def test_value(self):
        return self.value


class NumberParamField(ParamField):
    def __init__(self, key, minimum=0, maximum=100, enums=[], required=False, value=0):
        self.key = key
        self.enums = enums
        self.data_type = "number"
        self.minimum = minimum
        self.maximum = maximum
        self.value = value
        self.required = required

    def _schema(self):
        # todo enhance
        sc = {self.key: {"type": self.data_type}}
        if self.enums:
            sc[self.key]['enum'] = self.enums
        else:
            sc[self.key]['maximum'] = self.maximum
        return sc

    def _to_dict(self):
        if not self.enums:
            return {self.key: {
                "type": self.data_type,
                "minimum": self.minimum,
                "maximum": self.maximum
            }}
        return {self.key: {
            "type": self.data_type,
            "enum": self.enums
        }}

    @property
    def test_value(self):
        return self.value


class IntegerParamField(ParamField):
    def __init__(self, key, minimum=0, maximum=100, enums=[], required=False, value=0):
        self.enums = enums
        self.key = key
        self.data_type = "integer"
        self.minimum = minimum
        self.maximum = maximum
        self.required = required
        self.value = value

    def _schema(self):
        # todo enhance
        sc = {self.key: {"type": self.data_type}}
        if self.enums:
            sc[self.key]['enum'] = self.enums
        else:
            sc[self.key]['maximum'] = self.maximum
        return sc

    def _to_dict(self):
        if not self.enums:
            return {self.key: {
                "type": self.data_type,
                "minimum": self.minimum,
                "maximum": self.maximum
            }}
        return {self.key: {
            "type": self.data_type,
            "enum": self.enums
        }}

    @property
    def test_value(self):
        return self.value


class BooleanParamField(ParamField):
    def __init__(self, key, default=False, required=False):
        self.key = key
        self.data_type = "boolean"
        self.default = default
        self.required = required
        self.test_value = default

    def _schema(self):
        # todo enhance
        return {self.key: {"type": self.data_type, "default": self.default}}

    def _to_dict(self):
        return {self.key: {
            "type": self.data_type,
        }}


# metaclass是类的模板，所以必须从`type`类型派生：
class Metaclass(type):
    def __new__(cls, name, bases, attrs):
        if name == 'AseProto':
            return type.__new__(cls, name, bases, attrs)

        print('Found proto: %s' % name)
        mappings = dict()
        mappings['inputs'] = []
        mappings['accepts'] = []
        mappings['params'] = []
        for k, v in attrs.items():
            if k == "requestCls":
                for kk, vv in v.__class__.__dict__.items():
                    if isinstance(vv, PayloadField):
                        mappings['inputs'].append(vv)
                    if isinstance(vv, ParamField):
                        mappings['params'].append(vv)

            if k == "responseCls":
                for kk, vv in v.__class__.__dict__.items():
                    if kk.startswith("accept") or isinstance(vv, PayloadField):
                        mappings['accepts'].append(vv)
            if k == "call":
                mappings['call'] = v
            if k == "sub":
                mappings['sub'] = v
            if k == "route":
                mappings['route'] = v
            if k == "call_type":
                mappings['call_type'] = v

        attrs['__mappings__'] = mappings  # 保存属性和列的映射关系
        return type.__new__(cls, name, bases, attrs)


class WrapperBase(metaclass=Metaclass):
    config = {}

    def __init__(self):
        # 仅测试调试用
        self.inputs_test_values = {}
        self.params_test_values = {}
        self.respData = []
        self.session = SessionManager()

    def schema(self):
        s = Template(tpl)
        kwargs = self._parse_mapping()
        inputs_fields, inputs_body = self._parse_inputs()
        accepts_fields, accepts_payloads = self._parse_outputs()
        params_fields, required_params = self._parse_params()
        kwargs.update({"parameters": json.dumps(params_fields)})
        kwargs.update({"inputs_fields": json.dumps(inputs_fields)})
        kwargs.update({"accepts_fields": json.dumps(accepts_fields)})
        kwargs.update({"required_paramters": json.dumps(required_params)})
        kwargs.update({"inputs_payload": json.dumps(inputs_body)})
        kwargs.update({"accepts_payloads": json.dumps(accepts_payloads)})

        msg = {}
        try:
            msg = json.loads(s.render(**kwargs))
        except Exception as e:
            raise AttributeError("cant' format to schma %s" % str(e))

        print(json.dumps(msg, indent=4, ensure_ascii=False))
        return

    def _parse_inputs(self):
        inputs = self.__mappings__.get('inputs', [])
        if not inputs:
            raise Exception("your must specify at least one input field!")
        inputs_payloads = {}
        inputs_fields = {}
        for inp in inputs:
            if inp.data_type == IMAGE:
                inputs_fields.update({inp.key: {"dataType": "image"}})
                self.inputs_test_values.update({inp.key: inp.test_value})
                inputs_payloads.update(self._get_image_inputs_payload(inp.key))
            elif inp.data_type == AUDIO:
                inputs_fields.update({inp.key: {"dataType": "audio"}})
                self.inputs_test_values.update({inp.key: inp.test_value})
                inputs_payloads.update(self._get_audio_inputs_payload(inp.key))
            elif inp.data_type == STRING:
                inputs_fields.update({inp.key: {"dataType": "text"}})
                self.inputs_test_values.update({inp.key: inp.test_value})

                inputs_payloads.update(self._get_text_accepts_payload(inp.key))
        return inputs_fields, inputs_payloads

    def _parse_outputs(self):
        accepts = self.__mappings__.get('accepts', [])
        if not accepts:
            raise Exception("your must specify at least one accepts field!")
        accepts_payloads = {}
        accepts_fields = {}
        for acc in accepts:
            if acc.data_type == IMAGE:
                accepts_fields.update({acc.key: {"dataType": "image"}})
                accepts_payloads.update(self._get_image_inputs_payload(acc.key))
            elif acc.data_type == AUDIO:
                accepts_fields.update({acc.key: {"dataType": "audio"}})
                accepts_payloads.update(self._get_audio_inputs_payload(acc.key))
            elif acc.data_type == STRING:
                accepts_fields.update({acc.key: {"dataType": "text"}})
                accepts_payloads.update(self._get_text_accepts_payload(acc.key))
            ## binary support todo
        return accepts_fields, accepts_payloads

    def _parse_mapping(self):
        call = self.__mappings__.get("call", "atmos")
        call_type = self.__mappings__.get("call_type", "0")
        route = self.__mappings__.get("route", "/v1/private/{}".format(self.serviceId))
        sub = self.__mappings__.get("sub", "ase")
        hosts = self.__mappings__.get("hosts", "api.xf-yun.com")
        args = dict()
        args['serviceId'] = self.serviceId
        args['service'] = json.dumps([self.serviceId])
        args['version'] = self.version
        args['call'] = call
        args['call_type'] = call_type
        args['route'] = route
        args['sub'] = sub
        args['hosts'] = hosts
        return args

    def _parse_params(self):
        params = self.__mappings__.get('params', [])
        params_fields = {}
        required_params = []
        for param in params:
            if param.required:
                required_params.append(param.key)
            params_fields.update(param._schema())
            self.params_test_values.update()
            self.params_test_values[param.key] = param.test_value
        return params_fields, required_params

    def _parse_payload(self):
        pass

    def _get_image_inputs_payload(self, key):
        return {
            key: {
                "type": "object",
                "properties": {
                    "encoding": {
                        "type": "string",
                        "enum": ["jpg", "png", "bmp", "jpeg"]
                    },
                    "image": {
                        "type": "string",
                        "minLength": 1,
                        "maxLength": 10485760
                    },
                    "status": {
                        "type": "integer",
                        "enum": [3]
                    }
                }
            }
        }

    def _get_audio_inputs_payload(self, key):
        return {
            key: {
                "type": "object",
                "properties": {
                    "encoding": {
                        "type": "string",
                        "enum": [
                            "opus",
                            "speex"
                        ]
                    },
                    "sample_rate": {
                        "type": "integer",
                        "enum": [
                            8000,
                            16000
                        ]
                    },
                    "bit_depth": {
                        "type": "integer",
                        "enum": [
                            8,
                            16
                        ]
                    },
                    "channels": {
                        "type": "integer",
                        "enum": [
                            1,
                            2
                        ]
                    },
                    "status": {
                        "type": "integer",
                        "enum": [
                            0,
                            1,
                            2,
                            3
                        ]
                    },
                    "seq": {
                        "type": "integer"
                    },
                    "audio": {
                        "type": "string"
                    }
                }
            }
        }

    def _get_text_inputs_payload(self, key):
        return {
            key: {
                "type": "object",
                "properties": {
                    "encoding": {
                        "type": "string",
                        "enum": [
                            "utf8",
                            "gb2312"
                        ]
                    },
                    "status": {
                        "type": "integer",
                        "enum": [
                            0,
                            1,
                            2,
                            3
                        ]
                    },
                    "seq": {
                        "type": "integer"
                    },
                    "text": {
                        "type": "string"
                    }
                }
            }
        }

    def _get_text_accepts_payload(self, key):
        return {
            key: {
                "type": "object",
                "properties": {
                    "encoding": {
                        "type": "string",
                        "enum": [
                            "utf8",
                            "gb2312"
                        ]
                    },
                    "compress": {
                        "type": "string",
                        "enum": [
                            "gzip"
                        ]
                    },
                    "status": {
                        "type": "integer",
                        "enum": [
                            0,
                            1,
                            2,
                            3
                        ]
                    },
                    "seq": {
                        "type": "integer"
                    },
                    "text": {
                        "type": "string"
                    }
                }
            }
        }

    '''
    服务初始化
    @param config:
        插件初始化需要的一些配置，字典类型
        key: 配置名
        value: 配置的值
    @return
        ret: 错误码。无错误时返回0
    '''

    def wrapperInit(cls, config: {}) -> int:
        raise NotImplementedError("Please Inplement Wrapper Class Method: wrapperInit(cls, config: {}) ")

    '''
    服务逆初始化

    @return
        ret:错误码。无错误码时返回0
    '''

    def wrapperFini(cls) -> int:
        raise NotImplementedError("Please Inplement Wrapper Class Method: wrapperFini(cls) ")

    '''
    非会话模式计算接口,对应oneShot请求,可能存在并发调用

    @param usrTag 句柄
    #param params 功能参数
    @param  reqData     写入数据实体
    @param  respData    返回结果实体,内存由底层服务层申请维护,通过execFree()接口释放
    @param psrIds 需要使用的个性化资源标识列表
    @param psrCnt 需要使用的个性化资源个数

    @return 接口错误码
        reqDat
        ret:错误码。无错误码时返回0
    '''

    def wrapperOnceExec(cls, params: {}, reqData: DataListCls) -> Response:
        raise NotImplementedError(
            "Please Inplement Wrapper Class Method: wrapperOnceExec(cls, usrTag: str, params: {}, reqData: [], respData: [], psrIds: [], psrCnt: int) ")

    def wrapperOnceExecAsync(cls, params: {}, reqData: DataListCls, sid: str) -> Response:
        raise NotImplementedError(
            "Please Inplement Wrapper Class Method: wrapperOnceExec(cls, usrTag: str, params: {}, reqData: [], respData: [], psrIds: [], psrCnt: int) ")

    def wrapperError(cls, ret: int) -> str:
        if ret == 100:
            return "This is a  error return"
        return ""

    '''
        保留接口
    '''

    def wrapperCreate(cls, params: {}, sid: str) -> SessionCreateResponse:
        print(params)

        print(sid)
        return

    '''
        保留接口
    '''

    def wrapperWrite(cls, handle: str, datas: []) -> int:
        return 0

    '''
        保留接口
    '''

    def wrapperRead(cls, handle: str) -> []:
        return []

    def wrapperDestroy(cls, handle: str) -> int:
        return 0

    def check_resp(self, resp) -> bool:

        if not isinstance(resp, Response):
            log.error("Please return Response instance in function wrapperOnceExec")
            return False

        # check 响应list长度
        if len(resp.list) == 0:
            log.error("Please return content in response.list")
            return False

        # check 响应key是否重复
        keys = [r.key for r in resp.list]
        set_keys = set(keys)
        if not len(keys) == len(set_keys):
            log.error("response list keys must be unique...")
            log.error("invalid keys %s" % str(keys))
            return False

        print(keys)

    def run(self):
        # 1. 模拟调用初始化引擎
        #  传入配置当前模拟为空
        self.wrapperInit(self.config)

        try:
            # 2. 准备wrapperOnceExec需要的数据
            inputs_fields, inputs_body = self._parse_inputs()

            params_fields, required_params = self._parse_params()
            params = self.params_test_values
            reqData = []
            reqData.append(self.inputs_test_values)
            req = DataListCls()
            tmp = []
            for key, value in self.inputs_test_values.items():
                node = DataListNode()
                node.key = key
                node.data = value
                node.len = len(value)
                typeStr = inputs_fields[key]["dataType"]
                node.type = type_map.get(typeStr)
                tmp.append(node)

            req.list = tmp
            # 3. 模拟调用 exec，并返回数据
            response = self.wrapperOnceExec(params, req)
            self.check_resp(response)
        except Exception as e:
            # 4. 模拟检查 wrapperOnceExec返回
            log.error(e)
            self.wrapperError(-1)

        # todo respData 检查




#!/usr/bin/env python
# coding:utf-8
""" 
@author: nivic ybyang7
@license: Apache Licence 
@file: dto
@time: 2022/07/26
@contact: ybyang7@iflytek.com
@site:  
@software: PyCharm 

# code is far away from bugs with the god animal protecting
    I love animals. They taste delicious.
              ┏┓      ┏┓
            ┏┛┻━━━┛┻┓
            ┃      ☃      ┃
            ┃  ┳┛  ┗┳  ┃
            ┃      ┻      ┃
            ┗━┓      ┏━┛
                ┃      ┗━━━┓
                ┃  神兽保佑    ┣┓
                ┃　永无BUG！   ┏┛
                ┗┓┓┏━┳┓┏┛
                  ┃┫┫  ┃┫┫
                  ┗┻┛  ┗┻┛ 
"""

#  Copyright (c) 2022. Lorem ipsum dolor sit amet, consectetur adipiscing elit.
#  Morbi non lorem porttitor neque feugiat blandit. Ut vitae ipsum eget quam lacinia accumsan.
#  Etiam sed turpis ac ipsum condimentum fringilla. Maecenas magna.
#  Proin dapibus sapien vel ante. Aliquam erat volutpat. Pellentesque sagittis ligula eget metus.
#  Vestibulum commodo. Ut rhoncus gravida arcu.
from typing import Dict, Tuple, List, AnyStr
import json

from aiges.types import *



class DataListNode:
    def __init__(self):
        self.key: AnyStr = ''
        self.data: AnyStr = ''
        self.len = 0
        self.type = DataText


class DataListCls:
    def __init__(self):
        self.list: List[DataListNode]

    def get(self, key: str):
        for d in self.list:
            if d.key == key:
                return d
        return None


class ResponseData:
    def __init__(self, key='', data='', length=0, type=DataText, status=Once):
        self.key = key
        self.data = data
        self.len = length
        self.type = type
        self.status = status

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class Response:
    def __init__(self):
        self.list: List[ResponseData] = [ResponseData()]
        self.error_code = 0

    def response_err(self, error_code: int):
        self.error_code = error_code
        return self

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class SessionCreateResponse:
    def __init__(self):
        self.handle = ""
        self.error_code = 0

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)

if __name__ == '__main__':
    t = Response()
    d = ResponseData()
    d.len = 1
    t.list = [d]
    print(t.toJSON())



import json
import logging
import os
from typing import Dict

from jsonschema import validate
from jsonschema.exceptions import ValidationError

from seldon_core.metrics import split_image_tag

logger = logging.getLogger(__name__)

MODEL_IMAGE = os.environ.get("PREDICTIVE_UNIT_IMAGE")


class AigesInvalidMetadataError(Exception):
    pass


SELDON_ARRAY_SCHEMA = {
    "type": "object",
    "properties": {
        "messagetype": {"type": "string", "enum": ["tensor", "ndarray", "tftensor"]},
        "schema": {
            "type": "object",
            "properties": {
                "names": {"type": "array", "items": {"type": "string"}},
                "shape": {"type": "array", "items": {"type": "integer"}},
                # Shall we also include field for datatype (as in dtype for np.array)?
            },
            "additionalProperties": False,
        },
    },
    "required": ["messagetype"],
    "additionalProperties": False,
}

SELDON_JSON_SCHEMA = {
    "type": "object",
    "properties": {
        "messagetype": {"type": "string", "enum": ["jsonData"]},
        "schema": {"type": "object"},
    },
    "required": ["messagetype"],
    "additionalProperties": False,
}

SELDON_STR_SCHEMA = {
    "type": "object",
    "properties": {"messagetype": {"type": "string", "enum": ["strData"]}},
    "required": ["messagetype"],
    "additionalProperties": False,
}

SELDON_BIN_SCHEMA = {
    "type": "object",
    "properties": {"messagetype": {"type": "string", "enum": ["binData"]}},
    "required": ["messagetype"],
    "additionalProperties": False,
}


SELDON_CUSTOM_DEFINITION = {
    "type": "object",
    "properties": {
        "messagetype": {
            "type": "string",
            "not": {
                "enum": [
                    "tensor",
                    "ndarray",
                    "tftensor",
                    "jsonData",
                    "strData",
                    "binData",
                ]
            },
        },
        "schema": {"type": "object"},
    },
    "required": ["messagetype"],
    "additionalProperties": False,
}


TENSOR_DATA_TYPES = [
    "BOOL",
    "UINT8",
    "UINT16",
    "UINT32",
    "UINT64",
    "INT8",
    "INT16",
    "INT32",
    "INT64",
    "FP16",
    "FP32",
    "FP64",
    "BYTES",
]


METADATA_TENSOR_SCHEMA = {
    "type": "object",
    "properties": {
        "name": {"type": "string"},
        "datatype": {"type": "string", "enum": TENSOR_DATA_TYPES},
        "shape": {"type": "array", "items": {"type": "integer"}},
    },
    "additionalProperties": False,
}


INPUTS_OUTPUTS_SCHEMA = {
    "type": "array",
    "items": {
        "anyOf": [
            SELDON_ARRAY_SCHEMA,
            SELDON_JSON_SCHEMA,
            SELDON_STR_SCHEMA,
            SELDON_BIN_SCHEMA,
            SELDON_CUSTOM_DEFINITION,
            METADATA_TENSOR_SCHEMA,
        ]
    },
    "additionalProperties": False,
}


JSON_SCHEMA = {
    "type": "object",
    "properties": {
        "name": {"type": "string"},
        "versions": {"type": "array", "items": {"type": "string"}},
        "platform": {"type": "string"},
        "inputs": INPUTS_OUTPUTS_SCHEMA,
        "outputs": INPUTS_OUTPUTS_SCHEMA,
        "custom": {"type": "object", "additionalProperties": {"type": "string"}},
    },
    "additionalProperties": False,
}


def validate_model_metadata(data: Dict) -> Dict:
    """Validate metadata.

    Parameters
    ----------
    data
        User defined model metadata (json)

    Returns
    -------
        Validated model metadata (json)

    Raises
    ------
    SeldonInvalidMetadataError: if data cannot be properly validated

    Notes
    -----

    Read data from json and validate against v1 or v2 metadata schema.
    SeldonInvalidMetadataError exception will be raised if validation fails.
    """
    if MODEL_IMAGE is not None:
        image_name, image_version = split_image_tag(MODEL_IMAGE)
    else:
        image_name, image_version = "", ""

    default_meta = {
        "name": image_name,
        "versions": [image_version],
        "platform": "",
        "inputs": [],
        "outputs": [],
        "custom": {},
    }

    data = {**default_meta, **data}

    try:
        validate(data, JSON_SCHEMA)
    except ValidationError as e:
        raise SeldonInvalidMetadataError(e)

    logger.debug(f"Successfully validated metadata:\n{json.dumps(data, indent=2)}")
    return data



#!/usr/bin/env python
# coding:utf-8
""" 
@author: nivic ybyang7
@license: Apache Licence 
@file: package.py
@time: 2022/09/08
@contact: ybyang7@iflytek.com
@site:  
@software: PyCharm 

# code is far away from bugs with the god animal protecting
    I love animals. They taste delicious.
              ┏┓      ┏┓
            ┏┛┻━━━┛┻┓
            ┃      ☃      ┃
            ┃  ┳┛  ┗┳  ┃
            ┃      ┻      ┃
            ┗━┓      ┏━┛
                ┃      ┗━━━┓
                ┃  神兽保佑    ┣┓
                ┃　永无BUG！   ┏┛
                ┗┓┓┏━┳┓┏┛
                  ┃┫┫  ┃┫┫
                  ┗┻┛  ┗┻┛ 
"""

#  Copyright (c) 2022. Lorem ipsum dolor sit amet, consectetur adipiscing elit.
#  Morbi non lorem porttitor neque feugiat blandit. Ut vitae ipsum eget quam lacinia accumsan.
#  Etiam sed turpis ac ipsum condimentum fringilla. Maecenas magna.
#  Proin dapibus sapien vel ante. Aliquam erat volutpat. Pellentesque sagittis ligula eget metus.
#  Vestibulum commodo. Ut rhoncus gravida arcu.
import tarfile
import os
from aiges.utils.log import log


def reset(tarinfo):
    tarinfo.uid = tarinfo.gid = 0
    tarinfo.uname = tarinfo.gname = "root"
    return tarinfo


def pack(args):
    location = args.location
    files = checkget_project(location)

    output_dir = args.output
    make_tarfile("{}.tar.gz".format("pywrapper"),files)


def checkget_project(path):
    if not os.path.exists(path):
        raise FileNotFoundError

    basicfiles = ["wrapper.py", "requirements.txt", "Readme.md"]
    must_exists = False

    files = os.listdir(path)
    for fi in basicfiles:
        if fi == "wrapper.py":
            must_exists = True
        else:
            must_exists = False

        if fi not in files and must_exists:
            raise FileNotFoundError("Please execute pack in the wrapper.py dir.")
        elif not must_exists:
            log.warn("We suggest you should provide file: %s" % fi)
    return files

def make_tarfile(output_filename, files):
    with tarfile.open(output_filename, "w:gz") as tar:
        for fi in files:
            tar.add(fi, arcname=os.path.basename(fi))
            log.info("Archiving... {}".format(fi))
        log.info("Archived...{}".format(output_filename))


if __name__ == '__main__':
    make_tarfile("aiges.tar.gz", "demo")


#!/usr/bin/env python
# coding:utf-8
"""
@author: nivic ybyang7
@license: Apache Licence
@file: wrapper.py
@time: 2022/06/16
@contact: ybyang7@iflytek.com
@site:
@software: PyCharm

"""

#  Copyright (c) 2022. Lorem ipsum dolor sit amet, consectetur adipiscing elit.
#  Morbi non lorem porttitor neque feugiat blandit. Ut vitae ipsum eget quam lacinia accumsan.
#  Etiam sed turpis ac ipsum condimentum fringilla. Maecenas magna.
#  Proin dapibus sapien vel ante. Aliquam erat volutpat. Pellentesque sagittis ligula eget metus.
#  Vestibulum commodo. Ut rhoncus gravida arcu.
import sys

from sdk import WrapperBase, \
    StringParamField, \
    ImageBodyField, \
    StringBodyField
from mmocr.utils.ocr import MMOCR
import numpy as np
from PIL import Image
import io
import json


'''
定义请求类:
 params:  params 开头的属性代表最终HTTP协议中的功能参数parameters部分， 
          params Field支持 StringParamField，
          NumberParamField，BooleanParamField,IntegerParamField，每个字段均支持枚举
          params 属性多用于协议中的控制字段，请求body字段不属于params范畴

 input:    input字段多用与请求数据段，即body部分，当前支持 ImageBodyField, StringBodyField, 和AudioBodyField
'''


class UserRequest(object):
    params1 = StringParamField(key="p1", enums=["3", "eee"], value='3')
    params2 = StringParamField(key="p2", maxLength=44, required=True)
    params3 = StringParamField(key="p3", maxLength=44, required=False)

    input1 = ImageBodyField(key="data", path="test.png")
    input3 = ImageBodyField(key="data2", path="test.png")
    input2 = StringBodyField(key="switch", value="cc")


'''
定义响应类:
 accepts:  accepts代表响应中包含哪些字段, 以及数据类型

 input:    input字段多用与请求数据段，即body部分，当前支持 ImageBodyField, StringBodyField, 和AudioBodyField
'''


class UserResponse(object):
    accept1 = StringBodyField(key="boxes")
    accept2 = StringBodyField(key="boxes2")


'''
用户实现， 名称必须为Wrapper, 必须继承SDK中的 WrapperBase类
'''


model = None

class Wrapper(WrapperBase):
    serviceId = "mmocr"
    version = "backup.0"
    requestCls = UserRequest()
    responseCls = UserResponse()
    '''
    服务初始化
    @param config:
        插件初始化需要的一些配置，字典类型
        key: 配置名
        value: 配置的值
    @return
        ret: 错误码。无错误时返回0
    '''
    @classmethod
    def wrapperInit(cls, config: {}) -> int:
        print("Initializing ..")
        global model
        # Load models into memory
        model = MMOCR()
        return 0

    '''
    非会话模式计算接口,对应oneShot请求,可能存在并发调用
    @param usrTag 句柄
    @param params 功能参数
    @param  reqData     请求数据实体字段
    @param  respData    返回结果实体,内存由底层服务层申请维护,通过execFree()接口释放
    @param psrIds 需要使用的个性化资源标识列表
    @param psrCnt 需要使用的个性化资源个数
    @return 接口错误码
        reqDat
        ret:错误码。无错误码时返回0
    '''
    @classmethod
    def wrapperOnceExec(cls, usrTag: str, params: {}, reqData: [], respData: [], psrIds: [], psrCnt: int) -> int:
        img = np.array(Image.open(io.BytesIO(reqData[0]["data"])).convert('RGB'))
        global model
        rlt = model.readtext(img, details=True)
        rlt = json.dumps(rlt)
        respData.append({"key": "boxes", "data": rlt, "len": len(rlt), "status": 3, "type": 0})
        return 100

    '''
    服务逆初始化

    @return
        ret:错误码。无错误码时返回0
    '''

    @classmethod
    def wrapperFini(cls) -> int:
        return 0

    @classmethod
    def wrapperError(cls, ret: int) -> str:
        if ret == 100:
            return "user error defined here"
        return ""


if __name__ == '__main__':
    m = Wrapper()
    m.schema()
    m.run()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['aiges',
 'aiges.backup',
 'aiges.cmd',
 'aiges.core',
 'aiges.examples.once.mmocr',
 'aiges.examples.stream.mmocr',
 'aiges.examples.stream.mock',
 'aiges.utils']

package_data = \
{'': ['*'], 'aiges': ['test_data/*', 'tpls/*']}

install_requires = \
['jinja2>=2.0']

setup_kwargs = {
    'name': 'aiges',
    'version': '0.4.0',
    'description': "A module for test aiges's python wrapper.py",
    'long_description': None,
    'author': 'maybaby',
    'author_email': 'ybyang7@iflytek.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6',
}


setup(**setup_kwargs)
